/*

Portions of the code in this file Copyright (c) 2014 GitHub, Inc., used with
permission, under the following license:

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
 
*/

*{
	-moz-box-sizing:border-box;
	box-sizing:border-box
}

body{
	min-width:1020px;
	font:13px/1.4 Helvetica, arial, freesans, clean, sans-serif, "Segoe UI Emoji", "Segoe UI Symbol";
	color:#333;
	background-color:#fff
}

a{
	color:#4183c4;
	text-decoration:none
}

a:hover,a:focus,a:active{
	text-decoration:underline
}

h1,h2,h3,h4,h5,h6{
	margin-top:15px;
	margin-bottom:15px;
	line-height:1.1
}

ul,ol{
	padding:0;
	margin-top:0;
	margin-bottom:0
}

ol ol,ul ol{
	list-style-type:lower-roman
}

ul ul ol,ul ol ol,ol ul ol,ol ol ol{
	list-style-type:lower-alpha
}

tt,code{
	font:12px Consolas, "Liberation Mono", Menlo, Courier, monospace
}

pre{
	margin-top:0;
	margin-bottom:0;
	font:12px Consolas, "Liberation Mono", Menlo, Courier, monospace
}

.markdown-body{
	overflow:hidden;
	font-family:"Helvetica Neue", Helvetica, "Segoe UI", Arial, freesans, sans-serif;
	font-size:16px;
	line-height:1.6;
	word-wrap:break-word
}

.markdown-body>*:first-child{
	margin-top:0 !important
}

.markdown-body>*:last-child{
	margin-bottom:0 !important
}

.markdown-body .absent{
	color:#c00
}

.markdown-body .anchor{
	position:absolute;
	top:0;
	bottom:0;
	left:0;
	display:block;
	padding-right:6px;
	padding-left:30px;
	margin-left:-30px
}

.markdown-body .anchor:focus{
	outline:none
}

.markdown-body h1,.markdown-body h2,.markdown-body h3,.markdown-body h4,.markdown-body h5,.markdown-body h6{
	position:relative;
	margin-top:1em;
	margin-bottom:16px;
	font-weight:bold;
	line-height:1.4
}

.markdown-body h1 .octicon-link,.markdown-body h2 .octicon-link,.markdown-body h3 .octicon-link,.markdown-body h4 .octicon-link,.markdown-body h5 .octicon-link,.markdown-body h6 .octicon-link{
	display:none;
	color:#000;
	vertical-align:middle
}

.markdown-body h1:hover .anchor,.markdown-body h2:hover .anchor,.markdown-body h3:hover .anchor,.markdown-body h4:hover .anchor,.markdown-body h5:hover .anchor,.markdown-body h6:hover .anchor{
	height:1em;
	padding-left:8px;
	margin-left:-30px;
	line-height:1;
	text-decoration:none
}

.markdown-body h1:hover .anchor .octicon-link,.markdown-body h2:hover .anchor .octicon-link,.markdown-body h3:hover .anchor .octicon-link,.markdown-body h4:hover .anchor .octicon-link,.markdown-body h5:hover .anchor .octicon-link,.markdown-body h6:hover .anchor .octicon-link{
	display:inline-block
}

.markdown-body h1 tt,.markdown-body h1 code,.markdown-body h2 tt,.markdown-body h2 code,.markdown-body h3 tt,.markdown-body h3 code,.markdown-body h4 tt,.markdown-body h4 code,.markdown-body h5 tt,.markdown-body h5 code,.markdown-body h6 tt,.markdown-body h6 code{
	font-size:inherit
}

.markdown-body h1{
	padding-bottom:0.3em;
	font-size:2.25em;
	line-height:1.2;
	border-bottom:1px solid #eee
}

.markdown-body h2{
	padding-bottom:0.3em;
	font-size:1.75em;
	line-height:1.225;
	border-bottom:1px solid #eee
}

.markdown-body h3{
	font-size:1.5em;
	line-height:1.43
}

.markdown-body h4{
	font-size:1.25em
}

.markdown-body h5{
	font-size:1em
}

.markdown-body h6{
	font-size:1em;
	color:#777
}

.markdown-body p,.markdown-body blockquote,.markdown-body ul,.markdown-body ol,.markdown-body dl,.markdown-body table,.markdown-body pre{
	margin-top:0;
	margin-bottom:16px
}

.markdown-body hr{
	height:4px;
	padding:0;
	margin:16px 0;
	background-color:#e7e7e7;
	border:0 none
}

.markdown-body ul,.markdown-body ol{
	padding-left:2em
}

.markdown-body ul.no-list,.markdown-body ol.no-list{
	padding:0;
	list-style-type:none
}

.markdown-body ul ul,.markdown-body ul ol,.markdown-body ol ol,.markdown-body ol ul{
	margin-top:0;
	margin-bottom:0
}

.markdown-body li>p{
	margin-top:16px
}

.markdown-body dl{
	padding:0
}

.markdown-body dl dt{
	padding:0;
	margin-top:16px;
	font-size:1em;
	font-style:italic;
	font-weight:bold
}

.markdown-body dl dd{
	padding:0 16px;
	margin-bottom:16px
}

.markdown-body blockquote{
	padding:0 15px;
	color:#777;
	border-left:4px solid #ddd
}

.markdown-body blockquote>:first-child{
	margin-top:0
}

.markdown-body blockquote>:last-child{
	margin-bottom:0
}

.markdown-body table{
	display:block;
	width:100%;
	overflow:auto;
	word-break:normal;
	word-break:keep-all
}

.markdown-body table th{
	font-weight:bold
}

.markdown-body table th,.markdown-body table td{
	padding:6px 13px;
	border:1px solid #ddd
}

.markdown-body table tr{
	background-color:#fff;
	border-top:1px solid #ccc
}

.markdown-body table tr:nth-child(2n){
	background-color:#f8f8f8
}

.markdown-body img{
	max-width:100%;
	-moz-box-sizing:border-box;
	box-sizing:border-box
}

.markdown-body span.frame{
	display:block;
	overflow:hidden
}

.markdown-body span.frame>span{
	display:block;
	float:left;
	width:auto;
	padding:7px;
	margin:13px 0 0;
	overflow:hidden;
	border:1px solid #ddd
}

.markdown-body span.frame span img{
	display:block;
	float:left
}

.markdown-body span.frame span span{
	display:block;
	padding:5px 0 0;
	clear:both;
	color:#333
}

.markdown-body span.align-center{
	display:block;
	overflow:hidden;
	clear:both
}

.markdown-body span.align-center>span{
	display:block;
	margin:13px auto 0;
	overflow:hidden;
	text-align:center
}

.markdown-body span.align-center span img{
	margin:0 auto;
	text-align:center
}

.markdown-body span.align-right{
	display:block;
	overflow:hidden;
	clear:both
}

.markdown-body span.align-right>span{
	display:block;
	margin:13px 0 0;
	overflow:hidden;
	text-align:right
}

.markdown-body span.align-right span img{
	margin:0;
	text-align:right
}

.markdown-body span.float-left{
	display:block;
	float:left;
	margin-right:13px;
	overflow:hidden
}

.markdown-body span.float-left span{
	margin:13px 0 0
}

.markdown-body span.float-right{
	display:block;
	float:right;
	margin-left:13px;
	overflow:hidden
}

.markdown-body span.float-right>span{
	display:block;
	margin:13px auto 0;
	overflow:hidden;
	text-align:right
}

.markdown-body code,.markdown-body tt{
	padding:0;
	padding-top:0.2em;
	padding-bottom:0.2em;
	margin:0;
	font-size:85%;
	background-color:rgba(0,0,0,0.04);
	border-radius:3px
}

.markdown-body code:before,.markdown-body code:after,.markdown-body tt:before,.markdown-body tt:after{
	letter-spacing:-0.2em;
	content:"\00a0"
}

.markdown-body code br,.markdown-body tt br{
	display:none
}

.markdown-body del code{
	text-decoration:inherit;
	vertical-align:text-top
}

.markdown-body pre>code{
	padding:0;
	margin:0;
	font-size:100%;
	word-break:normal;
	white-space:pre;
	background:transparent;
	border:0
}

.markdown-body .highlight{
	margin-bottom:16px
}

.markdown-body .highlight pre,.markdown-body pre{
	padding:16px;
	overflow:auto;
	font-size:85%;
	line-height:1.45;
	background-color:#f7f7f7;
	border-radius:3px
}

.markdown-body .highlight pre{
	margin-bottom:0;
	word-break:normal
}

.markdown-body pre{
	word-wrap:normal
}

.markdown-body pre code,.markdown-body pre tt{
	display:inline;
	max-width:initial;
	padding:0;
	margin:0;
	overflow:initial;
	line-height:inherit;
	word-wrap:normal;
	background-color:transparent;
	border:0
}

.markdown-body pre code:before,.markdown-body pre code:after,.markdown-body pre tt:before,.markdown-body pre tt:after{
	content:normal
}

.container{
	width:790px;
	margin-right:auto;
	margin-left:auto
}

.container:before{
	display:table;
	content:""
}

.container:after{
	display:table;
	clear:both;
	content:""
}

.container.focus-content{
	width:620px
}

.container:first-child{
	margin-top:20px
}

.boxed-group{
	position:relative;
	border-radius:3px;
	margin-bottom:30px
}

.boxed-group>h3,.boxed-group .heading{
	background-color:#f5f5f5;
	margin:0;
	border-radius:3px 3px 0 0;
	border:1px solid #d8d8d8;
	border-bottom:0;
	padding:9px 10px 10px;
	font-size:14px;
	line-height:17px;
	display:block
}

#readme .markdown-body,#readme .plain{
	background-color:#fff;
	border:1px solid #ddd;
	border-bottom-left-radius:3px;
	border-bottom-right-radius:3px;
	padding:30px;
	word-wrap:break-word
}
